@@ -1,9 +1,9 @@
 # yiddish
 # A Python library for processing Yiddish text
-# Author: Isaac L. Bleaman (bleaman@berkeley.edu)
+# https://github.com/ibleaman/yiddish/
 
+import pkg_resources
 import re
-from urllib.request import urlopen
 import csv
 
 ##########
@@ -66,10 +66,10 @@ def strip_diacritics(string): # and replace with decomposed
 # import loshn-koydesh pronunciation list
 #########################################
 
-respellings_url = 'https://raw.githubusercontent.com/ibleaman/loshn-koydesh-pronunciation/master/orthographic-to-phonetic.txt'
-respellings_list = urlopen(respellings_url).read().decode('utf-8')
-respellings_list = respellings_list.split('\n')
-respellings_list = [line for line in respellings_list if line]
+respellings_path = pkg_resources.resource_filename('yiddish', 'submodules/loshn-koydesh-pronunciation/orthographic-to-phonetic.txt')
+with open(respellings_path, 'r') as file:
+    respellings_list = file.read().split('\n')
+    respellings_list = [line for line in respellings_list if line]
 
 lk = {} # orthographic to phonetic
 reverse_lk = {} # phonetic to orthographic
@@ -143,7 +143,7 @@ def transliterate(string, loshn_koydesh=False):
     romanized = replace_with_precombined(string)
     
     if loshn_koydesh:
-        tokens = re.findall(r"[אאַאָבבֿגדהוװוּױזחטייִײײַככּךלמםנןסעפּפֿףצץקרששׂתּת\-־]+|[^אאַאָבבֿגדהוװוּױזחטייִײײַככּךלמםנןסעפּפֿףצץקרששׂתּת\-־]", romanized)
+        tokens = re.findall(r"[אאַאָבבֿגדהוװוּױזחטייִײײַככּךלמםנןסעפּפֿףצץקרששׂתּת\-־']+|[^אאַאָבבֿגדהוװוּױזחטייִײײַככּךלמםנןסעפּפֿףצץקרששׂתּת\-־']", romanized)
         new_tokens = []
         for token in tokens:
             if token in lk and token not in germanic_semitic_homographs:
@@ -229,11 +229,11 @@ reverse_translit_table = [ # to precombined
     (r'ן(\'|")', r'נ\1'),
     (r'ף(\'|")', r'פֿ\1'),
     (r'ץ(\'|")', r'צ\1'),
-    (r'\bך', 'כ'), # no word-initial final letters
-    (r'\bם', 'מ'),
-    (r'\bן', 'נ'),
-    (r'\bף', 'פֿ'),
-    (r'\bץ', 'צ'),
+    (r'(?<!\')\bך', 'כ'), # no word-initial final letters (but skip forms like מיר'ן)
+    (r'(?<!\')\bם', 'מ'),
+    (r'(?<!\')\bן', 'נ'),
+    (r'(?<!\')\bף', 'פֿ'),
+    (r'(?<!\')\bץ', 'צ'),
 ]
 
 reverse_translit_exceptions = [
@@ -260,6 +260,7 @@ reverse_translit_exceptions = [
     (r'\bumeydel', 'אומאײדעל'), # אומאײדעלע
     (r'\bureynikl', 'אוראײניקל'),
     (r'\bbaayn', 'באַאײַנ'), # באַאײַנדרוקן, באַאײַנפֿלוסן
+    (r'\bbaumr', 'באַאומר'), # באַאומרויִקן
     (r'geayn', 'געאײַנ'), # געאײַנפֿלוסט
     (r'tsuayn', 'צואײַנ'),
     (r'durkhayl', 'דורכאײַל'), # דורכאײַלן
@@ -279,6 +280,7 @@ reverse_translit_exceptions = [
     (r'\bforur', 'פֿאָראור'), # פֿאָראורטל
     (r'\bfaribl', 'פֿאַראיבל'),
     (r'\bfarinteres', 'פֿאַראינטערעס'), # פֿאַראינטערעסירן
+    (r'\bvoltst', 'װאָלטסט'),
     
     # ay != ײַ
     (r'\brayon\b', 'ראַיאָן'),
@@ -296,6 +298,7 @@ reverse_translit_exceptions = [
     (r'\bkayor', 'קאַיאָר'),
     (r'\bayed', 'אַיעד'), # אַיעדער
     (r'\bayo\b', 'אַיאָ'),
+    (r'\bhayom\b', 'האַיאָם'), # needed for אַד־האַיאָם־האַזע -> עד־היום־הזה
     
     # ey != ײ
     (r'geyogt', 'געיאָגט'),
@@ -325,6 +328,7 @@ reverse_translit_exceptions = [
     (r'umloyal', 'אומלאָיאַל'),
     (r'loyal', 'לאָיאַל'),
     (r'paranoye', 'פּאַראַנאָיע'),
+    (r'poyezd', 'פּאָיעזד'),
     
     # ts != צ
     (r'tstu\b', 'טסטו'),
@@ -339,17 +343,27 @@ reverse_translit_exceptions = [
     (r'\bguts', 'גוטס'),
     (r'\bgeshefts', 'געשעפֿטס'),
     (r'(\b|ba|far|der)haltst', r'\1האַלטסט'),
+    (r'\bnentst', 'נענטסט'),
+    (r'\bneentst', 'נעענטסט'),
+    (r'\bnoentst', 'נאָענטסט'),
+    (r'\bshlekhts\b', 'שלעכטס'),
     (r'(\b|tse)shpaltst', r'\1שפּאַלטסט'),
     (r'(\b|tse|far)shpreytst', r'\1שפּרײטסט'),
     (r'shpetst', 'שפּעטסט'),
     (r'\brekhts\b', 'רעכטס'),
     (r'du shatst', 'דו שאַטסט'), # cf. ער שאַצט
     (r'\bforverts\b', 'פֿאָרװערטס'),
+    (r'\bayngemakhts', 'אײַנגעמאַכטס'),
+    (r'\bmasatshusets', 'מאַסאַטשוסעטס'),
+    (r'\bhundertst', 'הונדערטסט'),
     
     # kh != כ
     (r'\bpikhol', 'פּיקהאָל'), # פּיקהאָלץ, פּיקהאָלצן
     (r'\btsurikhalt', 'צוריקהאַלט'), # צוריקהאַלטן etc.
     (r'\bkrikhalt', 'קריקהאַלט'),
+    (r'\bshtokhol', 'שטאָקהאָל'), # שטאָקהאָלם/מער
+    (r'\bstokhol', 'סטאָקהאָל'),
+    (r'\bavekharg', 'אַװעקהאַרג'), # needed for אַװעקהרגע(נע)ן
     
     # sh != ש
     (r'\boysh(?!ers?\b|vits(er)?\b)', 'אױסה'), # the only exceptions to oysh = אױסה
@@ -365,22 +379,30 @@ semitic_germanic_homophones = [
     'אים',
     'בױ',
     'דאַן',
+    'דינע',
     'װײס',
     'װעסט',
+    'זאַל',
     'זאָל',
     'טאָמער',
     'טו',
+    'לױט',
     'לײען',
     'מאָגן',
+    'מאַסע',
     'מאַן',
     'מוטער',
     'מײַנע',
     'מע',
     'נעמען',
+    'עלעף',
     'עמער',
     'פּױלן',
+    'קאַשע',
     'קעלער',
     'קעץ',
+    'קערן',
+    'רוס',
     'שװאַך',
     'שיִער',
     'שנײ',
@@ -470,6 +492,7 @@ def respell_loshn_koydesh(text):
                 replacement = lk[key][1]
             else:
                 replacement = lk[key][0]
+            
                 
             # replace whole words (separated by spaces & punctuation, but not
             # followed by an apostrophe);
@@ -496,41 +519,95 @@ def respell_loshn_koydesh(text):
         text = re.sub(r'(?<![אאַאָבבֿגדהװווּזחטייִײײַױכּכךלמםנןסעפּפפֿףצץקרששׂתּת])' + key + r'(?![\'אאַאָבבֿגדהװווּזחטייִײײַױכּכךלמםנןסעפּפפֿףצץקרששׂתּת])', mistakes[key], text)
     
     return text
+
+#######################################################
+# convert phonetic spellings to loshn-koydesh spellings
+#######################################################
+
+# Note: input text WILL become precombined
+def spell_loshn_koydesh(text):
+    text = replace_with_precombined(text)
+    # loop over keys, in reverse order from longest keys to shortest
+    for key in sorted(list(reverse_lk.keys()), key=len, reverse=True):
+        # skip Germanic homophones
+        if key not in semitic_germanic_homophones:
+            replacement = reverse_lk[key]
+                
+            # replace whole words (separated by spaces & punctuation, but not
+            # followed by an apostrophe);
+            # also, append a Δ to the respelling so it's not accidentally overwritten
+            # (e.g., to avoid סעודה to סודע to סױדע)
+            text = re.sub(r'(?<![אאַאָבבֿגדהװווּזחטייִײײַױכּכךלמםנןסעפּפפֿףצץקרששׂתּתΔ])' + key + r'(?![\'אאַאָבבֿגדהװווּזחטייִײײַױכּכךלמםנןסעפּפפֿףצץקרששׂתּת])', 'Δ' + replacement, text)
     
+    # remove the added Δ
+    text = re.sub('Δ', '', text)
+
+    return text
+
 #######################################
 # convert YIVO orthography into Hasidic
 
 # note: all replacements are based on
 # looking for precombined characters
 #######################################
-hasidify_lexicon = 'https://raw.githubusercontent.com/ibleaman/hasidify_lexicon/master/'
+hasidify_lexicon_path = 'submodules/hasidify_lexicon'
 
-whole_word_variants = list(csv.reader(urlopen(hasidify_lexicon + 'whole_word_variants.csv').read().decode('utf-8').replace('\r', '').splitlines()))
-whole_word_variants = dict(zip([replace_with_precombined(row[0]) for row in whole_word_variants if row[0] != 'Find'], [replace_with_precombined(row[1]) for row in whole_word_variants if row[1] != 'Replace']))
+whole_word_variants = dict()
+with open(pkg_resources.resource_filename('yiddish', hasidify_lexicon_path + '/whole_word_variants.csv'), 'r') as file:
+    csv_reader = csv.DictReader(file)
+    for row in csv_reader:
+        whole_word_variants[replace_with_precombined(row['Find'])] = replace_with_precombined(row['Replace'])
 
-prefix_variants = list(csv.reader(urlopen(hasidify_lexicon + 'prefix_variants.csv').read().decode('utf-8').replace('\r', '').splitlines()))
-prefix_variants = dict(zip([replace_with_precombined(row[0]) for row in prefix_variants if row[0] != 'Find'], [replace_with_precombined(row[1]) for row in prefix_variants if row[1] != 'Replace']))
+prefix_variants = dict()
+with open(pkg_resources.resource_filename('yiddish', hasidify_lexicon_path + '/prefix_variants.csv'), 'r') as file:
+    csv_reader = csv.DictReader(file)
+    for row in csv_reader:
+        prefix_variants[replace_with_precombined(row['Find'])] = replace_with_precombined(row['Replace'])
 
-suffix_variants = list(csv.reader(urlopen(hasidify_lexicon + 'suffix_variants.csv').read().decode('utf-8').replace('\r', '').splitlines()))
-suffix_variants = dict(zip([replace_with_precombined(row[0]) for row in suffix_variants if row[0] != 'Find'], [replace_with_precombined(row[1]) for row in suffix_variants if row[1] != 'Replace']))
+suffix_variants = dict()
+with open(pkg_resources.resource_filename('yiddish', hasidify_lexicon_path + '/suffix_variants.csv'), 'r') as file:
+    csv_reader = csv.DictReader(file)
+    for row in csv_reader:
+        suffix_variants[replace_with_precombined(row['Find'])] = replace_with_precombined(row['Replace'])
 
-anywhere_variants = list(csv.reader(urlopen(hasidify_lexicon + 'anywhere_variants.csv').read().decode('utf-8').replace('\r', '').splitlines()))
-anywhere_variants = dict(zip([replace_with_precombined(row[0]) for row in anywhere_variants if row[0] != 'Find'], [replace_with_precombined(row[1]) for row in anywhere_variants if row[1] != 'Replace']))
+anywhere_variants = dict()
+with open(pkg_resources.resource_filename('yiddish', hasidify_lexicon_path + '/anywhere_variants.csv'), 'r') as file:
+    csv_reader = csv.DictReader(file)
+    for row in csv_reader:
+        anywhere_variants[replace_with_precombined(row['Find'])] = replace_with_precombined(row['Replace'])
 
-lkizmen = list(csv.reader(urlopen(hasidify_lexicon + 'lkizmen.csv').read().decode('utf-8').replace('\r', '').splitlines()))
-lkizmen = [replace_with_precombined(row[0]) for row in lkizmen if row[0] != 'Words']
+lkizmen = []
+with open(pkg_resources.resource_filename('yiddish', hasidify_lexicon_path + '/lkizmen.csv'), 'r') as file:
+    csv_reader = csv.reader(file)
+    header = next(csv_reader)  # Skip the header row
+    for row in csv_reader:
+        lkizmen.append(replace_with_precombined(row[0]))
 
-word_group_variants = list(csv.reader(urlopen(hasidify_lexicon + 'word_group_variants.csv').read().decode('utf-8').replace('\r', '').splitlines()))
-word_group_variants = dict(zip([replace_with_precombined(row[0]) for row in word_group_variants if row[0] != 'Find'], [replace_with_precombined(row[1]) for row in word_group_variants if row[1] != 'Replace']))
+word_group_variants = dict()
+with open(pkg_resources.resource_filename('yiddish', hasidify_lexicon_path + '/word_group_variants.csv'), 'r') as file:
+    csv_reader = csv.DictReader(file)
+    for row in csv_reader:
+        word_group_variants[replace_with_precombined(row['Find'])] = replace_with_precombined(row['Replace'])
 
-ik_exceptions = list(csv.reader(urlopen(hasidify_lexicon + 'ik_exceptions.csv').read().decode('utf-8').replace('\r', '').splitlines()))
-ik_exceptions = [replace_with_precombined(row[0]) for row in ik_exceptions if row[0] != 'Words']
+ik_exceptions = []
+with open(pkg_resources.resource_filename('yiddish', hasidify_lexicon_path + '/ik_exceptions.csv'), 'r') as file:
+    csv_reader = csv.reader(file)
+    header = next(csv_reader)  # Skip the header row
+    for row in csv_reader:
+        ik_exceptions.append(replace_with_precombined(row[0]))
 
-lekh_exceptions = list(csv.reader(urlopen(hasidify_lexicon + 'lekh_exceptions.csv').read().decode('utf-8').replace('\r', '').splitlines()))
-lekh_exceptions = [replace_with_precombined(row[0]) for row in lekh_exceptions if row[0] != 'Words']
+lekh_exceptions = []
+with open(pkg_resources.resource_filename('yiddish', hasidify_lexicon_path + '/lekh_exceptions.csv'), 'r') as file:
+    csv_reader = csv.reader(file)
+    header = next(csv_reader)  # Skip the header row
+    for row in csv_reader:
+        lekh_exceptions.append(replace_with_precombined(row[0]))
 
-last_minute_fixes = list(csv.reader(urlopen(hasidify_lexicon + 'last_minute_fixes.csv').read().decode('utf-8').replace('\r', '').splitlines()))
-last_minute_fixes = dict(zip([replace_with_precombined(row[0]) for row in last_minute_fixes if row[0] != 'Find'], [replace_with_precombined(row[1]) for row in last_minute_fixes if row[1] != 'Replace']))
+last_minute_fixes = dict()
+with open(pkg_resources.resource_filename('yiddish', hasidify_lexicon_path + '/last_minute_fixes.csv'), 'r') as file:
+    csv_reader = csv.DictReader(file)
+    for row in csv_reader:
+        last_minute_fixes[replace_with_precombined(row['Find'])] = replace_with_precombined(row['Replace'])
 
 reformatting = [
     ('וּװוּ', 'ואוואו'),
@@ -612,3 +689,40 @@ def hasidify(text):
     
     return text
     
+def desovietify(text):
+    text = replace_with_precombined(text)
+    text = re.split(r"([^אאַאָבבֿגדהווּװױזחטייִײײַכּכךלמםנןסעפפּפֿףצץקרששׂתּתA-Za-z'])", text)
+    
+    # add 'Γ' as a word/token boundary symbol
+    text = 'Γ'.join(text)
+    text = 'Γ' + text + 'Γ'
+
+    # replace unpointed alef with pasekh alef when not followed by vowels. 
+    # (unpointed alef, if not followed by a vov/yud-based vowel, is alway pasekh alef in Soviet orthography)
+    text = re.sub('א(?![י|יִ|ײ|ײַ|וּ|ױ|ו])', 'אַ', text)
+
+    # replace unpointed pey with fey
+    text = re.sub('פ', 'פֿ', text)
+
+    # replace final kof, mem, nun, tsadek, fey with long forms
+    text = re.sub('כ(?=Γ)', 'ך', text)
+    text = re.sub('מ(?=Γ)', 'ם', text)
+    text = re.sub('נ(?=Γ)', 'ן', text)
+    text = re.sub('צ(?=Γ)', 'ץ', text)
+    text = re.sub('פֿ(?=Γ)', 'ף', text)
+
+    # replace oyf, bay
+    text = re.sub('(?<=Γ)אַף(?=Γ)', 'אױף', text)
+    text = re.sub('(?<=Γ)אַפֿן(?=Γ)', 'אױפֿן', text)
+    text = re.sub('(?<=Γ)אוף(?=Γ)', 'אױף', text)
+    text = re.sub('(?<=Γ)אופֿ', 'אױפֿ', text)
+    text = re.sub('(?<=Γ)באַ(?=Γ)', 'בײַ', text)
+    text = re.sub('(?<=Γ)באַם(?=Γ)', 'בײַם', text)
+
+    text = spell_loshn_koydesh(text)
+
+    # remove Greek letters
+    text = text.replace('Δ', '')
+    text = text.replace('Γ', '')
+
+    return text
